(**************************************************************************)
(*                                                                        *)
(*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       *)
(*  Kim Nguyen, Laurent Sartre                                            *)
(*                                                                        *)
(*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      *)
(*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 *)
(*                                                                        *)
(*  https://www.informatique-mpi.fr/                                      *)
(*                                                                        *)
(**************************************************************************)

(* Recherche d'un motif dans un texte avec l'algorithme de Rabin--Karp *)

val rabin_karp: string -> string -> int
  (* `rabin_karp m t` affiche toutes les occurrences de `m` dans `t` et
     renvoie leur nombre (mais si `m` est la chaîne vide, renvoie le
     nombre sans rien afficher) *)

val hash: string -> int -> int -> int
  (* `hash s i len` renvoie le hachage de `s[i..i+len[` *)

(* Pour chercher les occurrences d'un motif dans un texte que l'on
   décompresse (par exemple avec l'algorithme LZW, cf lzw.ml), sans
   stocker tout le texte décompressé.

   L'idée est de ne conserver que les M derniers caractères décompressés
   dans la structure suivante. *)
module Buffer : sig

  type t

  val create: string -> t
  (* Crée une structure pour un motif donné, initialement vide de tout
     caractère. Le motif doit être non vide. *)

  val add_string: t -> string -> unit
  (* Ajoute des caractères et signale toute occurrence du motif. *)

  val count: t -> int
  (* Renvoie le nombre total d'occurrences détectées jusqu'à présent. *)

end
